
****************************************************************************
*   This file contains a function to decide which phase space should be
*   taken in case of multi-channels.
c***************************************************************************
c   LIST OF ALL FUNCTIONS AND SUBROUTINES IN THIS FILE:
c
c     LOGICAL FUNCTION Choose_PS(PS_number, v, L)
c     LOGICAL FUNCTION VA_Choose(PS_number, mom, L, bos1)
c     LOGICAL FUNCTION VVA_Choose(PS_number, mom, L, bos1, bos2)
c     LOGICAL FUNCTION WZA_Choose_anom(PS_number, mom, L, bos1, bos2)
c     LOGICAL FUNCTION ZZA_Choose_anom(PS_number, mom, L, bos1, bos2)
c     LOGICAL FUNCTION VAA_Choose(PS_number, mom, L, bos1)
c     LOGICAL FUNCTION WAA_Choose_anom(PS_number, mom, L, bos1)
c     LOGICAL FUNCTION ZAA_Choose_anom(PS_number, mom, L, bos1)
c     LOGICAL FUNCTION ZnAA_Choose_anom(PS_number, mom, L, bos1)
c     LOGICAL FUNCTION AAA_Choose_anom(PS_number, mom, L, bos1)
c     LOGICAL FUNCTION VAJ_Choose(PS_number, mom)
c
c***************************************************************************
*   Vera Hankele, <vera@particle.uni-karlsruhe.de>
*   Initial version:  09 Februar 2009
*   Last modified: 04.07.2012 by Oliver Schlimpert
****************************************************************************

c*************************************************************************
      LOGICAL FUNCTION Choose_PS(PS_number, v, L)
c*************************************************************************
c     This is the general routine to choose a certain phase space generator.
c*************************************************************************
c     INPUT 
c     PS_number:  Number of phase space generator used to generate this point
c     v    :      4-momenta of the particles, which are no jets. 
c     L    :      id for momentum set
c
c     OUTPUT
c     Cuts   : .true. or .false. depending on whether this phase space
c              generator is selected or not.
c*************************************************************************

      IMPLICIT NONE

#include "global.inc"
#include "process.inc"

      integer ps_number

c     input variables
      double precision v(0:3,max_v,max_kin) 
      integer L

c     external functions
      logical VA_Choose, VVA_Choose, VAA_Choose, VAJ_Choose 
      logical WZA_Choose_anom, ZZA_Choose_anom, WAA_Choose_anom
      logical ZAA_Choose_anom, ZnAA_Choose_anom, AAA_Choose_anom
      external VA_Choose, VVA_Choose, VAA_Choose, VAJ_Choose
      external WZA_Choose_anom, ZZA_Choose_anom, WAA_Choose_anom 
      external ZAA_Choose_anom, ZnAA_Choose_anom, AAA_Choose_anom

c For most processes no multi-channel approach is used -> .TRUE.
      Choose_PS = .true.

      SELECT CASE(procID)
      CASE(WPA,WMA)! charge of W does not matter
!          Choose_PS = VA_Choose(PS_number, v(0,1,L), L , 3)
         Choose_PS = VAJ_Choose(PS_number, v(0,1,L))
      CASE(ZA)
         Choose_PS = VA_Choose(PS_number, v(0,1,L), L , 2)
      CASE(WWA)
         Choose_PS = VVA_Choose(PS_number, v(0,1,L), L , 3, 3)
      CASE(ZZA)
         if(with_anom) then
            Choose_PS = ZZA_Choose_anom(PS_number, v(0,1,L), L , 2, 2)
         else
            Choose_PS = VVA_Choose(PS_number, v(0,1,L), L , 2, 2)
         endif
      CASE(WPZA,WMZA)  ! charge of W does not matter
         if(with_anom) then
            Choose_PS = WZA_Choose_anom(PS_number, v(0,1,L), L , 3, 2)
         else
            Choose_PS = VVA_Choose(PS_number, v(0,1,L), L , 3, 2)
         endif
      CASE(WPAA,WMAA,WPAAj,WMAAj)  ! charge of W does not matter
         if(with_anom) then
            Choose_PS = WAA_Choose_anom(PS_number, v(0,1,L), L , 3)
         else
            Choose_PS = VAA_Choose(PS_number, v(0,1,L), L , 3)
         endif
      CASE(ZAA)  
         if(with_anom) then
            Choose_PS = ZAA_Choose_anom(PS_number, v(0,1,L), L , 2)
         else
            Choose_PS = VAA_Choose(PS_number, v(0,1,L), L , 2)
         endif
      CASE(ZnAA)  
         if(with_anom) then
            Choose_PS = ZnAA_Choose_anom(PS_number, v(0,1,L), L , 2)
         else
            Choose_PS = .true.
         endif
      CASE(AAA)  
         if(with_anom) then
            Choose_PS = AAA_Choose_anom(PS_number, v(0,1,L), L , 1)
         else
            Choose_PS = .true.
         endif
      CASE(WMAJ,WPAJ)
         Choose_PS = VAJ_Choose(PS_number, v(0,1,L))
      END SELECT

      RETURN
      END



c*************************************************************************
      LOGICAL FUNCTION VA_Choose(PS_number, mom, L, bos1)
c*************************************************************************
c     This routine selects the right phase space generator for VA
c     processes
c*************************************************************************
      IMPLICIT NONE

c     input variables
      integer PS_number, L
      double precision mom(0:3,3)
      integer bos1

c local variables
      double precision tempvector(0:4)
      double precision tempva(0:4)
      double precision choosev1,chooseva
      integer mu, myps
      logical VA_ps

c     external functions
      double precision dotrr
      external dotrr


      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)



      VA_ps = .true.

      do mu=0,3
         tempvector(mu)=mom(mu,1)+mom(mu,2)
         tempva(mu)=tempvector(mu)+mom(mu,3)
      enddo
      tempvector(4) = dotrr(tempvector,tempvector)
      tempva(4) = dotrr(tempva,tempva)

      choosev1=abs(sqrt(tempvector(4))-sqrt(xm2(bos1)))
      chooseva=abs(sqrt(tempva(4))-sqrt(xm2(bos1)))

      if(chooseva.le.choosev1) then
        myps = 1
      else 
        myps = 2
      endif
      if (ps_number.ne.myps) VA_ps=.false.

      VA_Choose = VA_ps
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION VVA_Choose(PS_number, mom, L, bos1, bos2)
c*************************************************************************
c     This routine selects the right phase space generator for VVA
c     processes
c*************************************************************************
      IMPLICIT NONE

#include "global.inc"

      integer ps_number

c     input variables
      integer L
      double precision mom(0:3,5)
      integer bos1, bos2

c local variables
      double precision tempplus(0:3),tempminus(0:3)
      double precision tempp(0:3),tempm(0:3)
      double precision choosep,choosem
      integer mu
      logical VVA_ps

c     external functions
      double precision dotrr
      external dotrr

      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)

      VVA_ps = .true.

      do mu=0,3
         tempplus(mu)=mom(mu,1)+mom(mu,2)
         tempminus(mu)=mom(mu,3)+mom(mu,4)
         tempp(mu)=tempplus(mu)+mom(mu,5)
         tempm(mu)=tempminus(mu)+mom(mu,5)
      enddo

      choosep=(sqrt(dotrr(tempp,tempp))-sqrt(xm2(bos1)))
      choosem=(sqrt(dotrr(tempm,tempm))-sqrt(xm2(bos2)))


      if (ps_number.eq.1) then
         if(choosep.lt.30.or.choosem.lt.30) VVA_ps=.false.
      elseif (ps_number.eq.2) then
         if(choosep.gt.30.or.choosep.gt.choosem) VVA_ps=.false.
      elseif (ps_number.eq.3) then
         if(choosem.gt.30.or.choosem.gt.choosep) VVA_ps=.false.
      endif

      VVA_Choose = VVA_ps
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION WZA_Choose_anom(PS_number, mom, L, bos1, bos2)
c*************************************************************************
c     This routine selects the right phase space generator for the WZA
c     process with anomalous couplings 
c*************************************************************************
      IMPLICIT NONE

#include "global.inc"
#include "coupl.inc"

      integer ps_number

c     input variables
      integer L
      double precision mom(0:3,5)
      integer bos1, bos2

c local variables
      double precision tempplus(0:3),tempminus(0:3)
      double precision tempp(0:3),tempm(0:3)
      double precision choosep,choosem,chooseh
      integer mu
      logical WZA_ps

c     external functions
      double precision dotrr
      external dotrr

      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)

      WZA_ps = .true.

      do mu=0,3
         tempplus(mu)=mom(mu,1)+mom(mu,2)
         tempminus(mu)=mom(mu,3)+mom(mu,4)
         tempp(mu)=tempplus(mu)+mom(mu,5)
         tempm(mu)=tempminus(mu)+mom(mu,5)       
      enddo

      choosep=abs(sqrt(dotrr(tempp,tempp))-sqrt(xm2(bos1)))
      choosem=abs(sqrt(dotrr(tempm,tempm))-sqrt(xm2(bos2)))
      chooseh=abs(sqrt(dotrr(tempm,tempm))-sqrt(xm2(6)))

      if (ps_number.eq.1) then
         if(choosep.lt.30.or.choosem.lt.30.or.chooseh.le.7.0D-2) WZA_ps=.false.
      elseif (ps_number.eq.2) then
         if((choosep.gt.30.or.choosep.gt.choosem).or.chooseh.le.7.0D-2) WZA_ps=.false.
      elseif (ps_number.eq.3) then
         if((choosem.gt.30.or.choosem.gt.choosep).and.chooseh.gt.7.0D-2) WZA_ps=.false.
      endif

      WZA_Choose_anom = WZA_ps
      
      RETURN
      END


c*************************************************************************
      LOGICAL FUNCTION ZZA_Choose_anom(PS_number, mom, L, bos1, bos2)
c*************************************************************************
c     This routine selects the right phase space generator for ZZA
c     with anomalous couplings
c*************************************************************************
      IMPLICIT NONE

#include "global.inc"
#include "coupl.inc"

      integer ps_number

c     input variables
      integer L
      double precision mom(0:3,5)
      integer bos1, bos2

c local variables
      double precision tempplus(0:3),tempminus(0:3)
      double precision tempp(0:3),tempm(0:3),temppm(0:3)
      double precision choosep,choosem,chooseh1,chooseh2,chooseh3
      integer mu, myps
      logical ZZA_ps

c     external functions
      double precision dotrr
      external dotrr

      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)

      ZZA_ps = .true.

      do mu=0,3
         tempplus(mu)=mom(mu,1)+mom(mu,2)
         tempminus(mu)=mom(mu,3)+mom(mu,4)
         temppm(mu)=tempplus(mu)+tempminus(mu)
         tempp(mu)=tempplus(mu)+mom(mu,5)
         tempm(mu)=tempminus(mu)+mom(mu,5)       
      enddo

      choosep=abs(sqrt(dotrr(tempp,tempp))-sqrt(xm2(bos1)))
      choosem=abs(sqrt(dotrr(tempm,tempm))-sqrt(xm2(bos2)))
      chooseh1=abs(sqrt(dotrr(temppm,temppm))-sqrt(xm2(6)))
      chooseh2=abs(sqrt(dotrr(tempp,tempp))-sqrt(xm2(6)))
      chooseh3=abs(sqrt(dotrr(tempm,tempm))-sqrt(xm2(6)))


      if((choosep.lt.30.and.choosep.lt.choosem).or.(chooseh2.le.7.0D-2.and.chooseh2.lt.min(chooseh1,chooseh3))) then
         myps=2
      elseif((choosem.lt.30.and.choosem.lt.choosep).or. 
     1      (chooseh3.le.7.0D-2.and.chooseh3.lt.min(chooseh1,chooseh2))) then
         myps=3
      else
         myps=1 
      endif 

      if (ps_number.ne.myps) ZZA_ps=.false.

      ZZA_Choose_anom = ZZA_ps
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION VAA_Choose(PS_number, mom, L, bos1)
c*************************************************************************
c     This routine selects the right phase space generator for VAA
c     processes
c*************************************************************************
      IMPLICIT NONE

c     input variables
      integer PS_number, L
      double precision mom(0:3,4)
      integer bos1

c local variables
      double precision tempvector(0:4)
      double precision tempv2(0:4), tempv3(0:4), tempva(0:4)
      double precision choosev1,choosev2,choosev3,chooseva
      integer mu, myps
      logical VAA_ps

c     external functions
      double precision dotrr
      external dotrr


      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)



      VAA_ps = .true.

      do mu=0,3
         tempvector(mu)=mom(mu,1)+mom(mu,2)
         tempv2(mu)=tempvector(mu)+mom(mu,3)
         tempv3(mu)=tempvector(mu)+mom(mu,4)
         tempva(mu)=tempv2(mu)+mom(mu,4)
      enddo
      tempvector(4) = dotrr(tempvector,tempvector)
      tempv2(4) = dotrr(tempv2,tempv2)
      tempv3(4) = dotrr(tempv3,tempv3)
      tempva(4) = dotrr(tempva,tempva)

      choosev1=abs(sqrt(tempvector(4))-sqrt(xm2(bos1)))
      choosev2=abs(sqrt(tempv2(4))-sqrt(xm2(bos1)))
      choosev3=abs(sqrt(tempv3(4))-sqrt(xm2(bos1)))
      chooseva=abs(sqrt(tempva(4))-sqrt(xm2(bos1)))

      if(choosev2.le.min(choosev1,choosev3,chooseva)) then
        myps = 2
      else if(choosev3.le.min(choosev1,choosev2,chooseva)) then
        myps = 3
      else if(chooseva.le.min(choosev1,choosev2,choosev3)) then
        myps = 4
      else 
        myps = 1
      endif
      if (ps_number.ne.myps) VAA_ps=.false.

      VAA_Choose = VAA_ps
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION WAA_Choose_anom(PS_number, mom, L, bos1)
c*************************************************************************
c     This routine selects the right phase space generator for WAA
c     with anomalous couplings
c*************************************************************************
      IMPLICIT NONE

c     input variables
      integer PS_number, L
      double precision mom(0:3,4)
      integer bos1

c local variables
      double precision tempvector(0:4)
      double precision tempv2(0:4), tempv3(0:4), tempva(0:4), tempaa(0:4)
      double precision choosev1,choosev2,choosev3,chooseva,chooseaa
      integer mu, myps
      logical WAA_ps

c     external functions
      double precision dotrr
      external dotrr


      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)



      WAA_ps = .true.

      do mu=0,3
         tempvector(mu)=mom(mu,1)+mom(mu,2)
         tempv2(mu)=tempvector(mu)+mom(mu,3)
         tempv3(mu)=tempvector(mu)+mom(mu,4)
         tempva(mu)=tempv2(mu)+mom(mu,4)
         tempaa(mu)=mom(mu,3)+mom(mu,4)
      enddo
      tempvector(4) = dotrr(tempvector,tempvector)
      tempv2(4) = dotrr(tempv2,tempv2)
      tempv3(4) = dotrr(tempv3,tempv3)
      tempva(4) = dotrr(tempva,tempva)
      tempaa(4) = dotrr(tempaa,tempaa)

      choosev1=abs(sqrt(tempvector(4))-sqrt(xm2(bos1)))
      choosev2=abs(sqrt(tempv2(4))-sqrt(xm2(bos1)))
      choosev3=abs(sqrt(tempv3(4))-sqrt(xm2(bos1)))
      chooseva=abs(sqrt(tempva(4))-sqrt(xm2(bos1)))
      chooseaa=abs(sqrt(tempaa(4))-sqrt(xm2(6))) !for H -> A A

      if((choosev2.le.min(choosev1,choosev3,chooseva)).and.(chooseaa.gt.7.0D-2)) then
        myps = 2
      else if((choosev3.le.min(choosev1,choosev2,chooseva)).and.(chooseaa.gt.7.0D-2)) then
        myps = 3
      else if((chooseva.le.min(choosev1,choosev2,choosev3)).and.(chooseaa.gt.7.0D-2)) then
        myps = 4
      else if(chooseaa.le.7.0D-2) then
        myps = 5
      else 
        myps = 1
      endif
      if (ps_number.ne.myps) WAA_ps=.false.

      WAA_Choose_anom = WAA_ps
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION ZAA_Choose_anom(PS_number, mom, L, bos1)
c*************************************************************************
c     This routine selects the right phase space generator for ZAA
c     with anomalous couplings
c*************************************************************************
      IMPLICIT NONE

c     input variables
      integer PS_number, L
      double precision mom(0:3,4)
      integer bos1

c local variables
      double precision tempvector(0:4)
      double precision tempv2(0:4), tempv3(0:4), tempva(0:4), tempaa(0:4)
      double precision choosev1,choosev2,choosev3,chooseva,choosehaa,choosehv2,choosehv3
      integer mu, myps
      logical ZAA_ps

c     external functions
      double precision dotrr
      external dotrr


      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)



      ZAA_ps = .true.

      do mu=0,3
         tempvector(mu)=mom(mu,1)+mom(mu,2)
         tempv2(mu)=tempvector(mu)+mom(mu,3)
         tempv3(mu)=tempvector(mu)+mom(mu,4)
         tempva(mu)=tempv2(mu)+mom(mu,4)
         tempaa(mu)=mom(mu,3)+mom(mu,4)
      enddo
      tempvector(4) = dotrr(tempvector,tempvector)
      tempv2(4) = dotrr(tempv2,tempv2)
      tempv3(4) = dotrr(tempv3,tempv3)
      tempva(4) = dotrr(tempva,tempva)
      tempaa(4) = dotrr(tempaa,tempaa)

      choosev1=abs(sqrt(tempvector(4))-sqrt(xm2(bos1)))
      choosev2=abs(sqrt(tempv2(4))-sqrt(xm2(bos1)))
      choosev3=abs(sqrt(tempv3(4))-sqrt(xm2(bos1)))
      chooseva=abs(sqrt(tempva(4))-sqrt(xm2(bos1)))
      choosehaa=abs(sqrt(tempaa(4))-sqrt(xm2(6))) ! for H -> AA
      choosehv2=abs(sqrt(tempv2(4))-sqrt(xm2(6))) ! for H -> ZA1
      choosehv3=abs(sqrt(tempv3(4))-sqrt(xm2(6))) ! for H -> ZA2

      if(((choosev2.le.min(choosev1,choosev3,chooseva)).and.(min(choosehaa,choosehv2,choosehv3).gt.7.0D-2))
     1   .or.((choosehv2.le.7.0D-2).and.(choosehv2.lt.min(choosehaa,choosehv3)))) then
        myps = 2
      else if(((choosev3.le.min(choosev1,choosev2,chooseva)).and.(min(choosehaa,choosehv2,choosehv3).gt.7.0D-2))
     1       .or.((choosehv3.le.7.0D-2).and.(choosehv3.lt.min(choosehaa,choosehv2)))) then
        myps = 3
      else if((chooseva.le.min(choosev1,choosev2,choosev3)).and.(min(choosehaa,choosehv2,choosehv3).gt.7.0D-2)) then
        myps = 4
      else if((choosehaa.le.7.0D-2).and.(choosehaa.le.min(choosehv2,choosehv3))) then
        myps = 5
      else 
        myps = 1
      endif
      if (ps_number.ne.myps) ZAA_ps=.false.

      ZAA_Choose_anom = ZAA_ps
      
      RETURN
      END


c*************************************************************************
      LOGICAL FUNCTION ZnAA_Choose_anom(PS_number, mom, L, bos1)
c*************************************************************************
c     This routine selects the right phase space generator for ZAA (Z->nu nu~)
c     with anomalous couplings
c*************************************************************************
      IMPLICIT NONE

c     input variables
      integer PS_number, L
      double precision mom(0:3,4)
      integer bos1

c local variables
      double precision tempvector(0:4)
      double precision tempv2(0:4), tempv3(0:4), tempva(0:4), tempaa(0:4)
      double precision choosev1,choosehaa,choosehv2,choosehv3
      integer mu, myps
      logical ZnAA_ps

c     external functions
      double precision dotrr
      external dotrr


      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)



      ZnAA_ps = .true.

      do mu=0,3
         tempvector(mu)=mom(mu,1)+mom(mu,2)
         tempv2(mu)=tempvector(mu)+mom(mu,3)
         tempv3(mu)=tempvector(mu)+mom(mu,4)
         tempva(mu)=tempv2(mu)+mom(mu,4)
         tempaa(mu)=mom(mu,3)+mom(mu,4)
      enddo
      tempvector(4) = dotrr(tempvector,tempvector)
      tempv2(4) = dotrr(tempv2,tempv2)
      tempv3(4) = dotrr(tempv3,tempv3)
      tempva(4) = dotrr(tempva,tempva)
      tempaa(4) = dotrr(tempaa,tempaa)

      choosev1=abs(sqrt(tempvector(4))-sqrt(xm2(bos1)))
      choosehaa=abs(sqrt(tempaa(4))-sqrt(xm2(6))) ! for H -> AA
      choosehv2=abs(sqrt(tempv2(4))-sqrt(xm2(6))) ! for H -> ZA1
      choosehv3=abs(sqrt(tempv3(4))-sqrt(xm2(6))) ! for H -> ZA2

      if(choosehv2.le.min(choosehaa,choosehv3).and.choosehv2.le.7.0D-2) then
        myps = 2
      else if(choosehv3.le.min(choosehaa,choosehv2).and.choosehv3.le.7.0D-2) then
        myps = 3
      else if(choosehaa.le.min(choosehv2,choosehv3).and.choosehaa.le.7.0D-2) then
        myps = 4
      else 
        myps = 1
      endif
      if (ps_number.ne.myps) ZnAA_ps=.false.

      ZnAA_Choose_anom = ZnAA_ps
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION AAA_Choose_anom(PS_number, mom, L, bos1)
c*************************************************************************
c     This routine selects the right phase space generator for AAA 
c     with anomalous couplings
c*************************************************************************
      IMPLICIT NONE

c     input variables
      integer PS_number, L
      double precision mom(0:3,3)
      integer bos1

c local variables
      double precision tempv12(0:4), tempv13(0:4), tempv23(0:4)
      double precision chooseh12,chooseh13,chooseh23
      integer mu, myps
      logical AAA_ps

c     external functions
      double precision dotrr
      external dotrr


      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)



      AAA_ps = .true.

      do mu=0,3
         tempv12(mu)=mom(mu,1)+mom(mu,2)
         tempv13(mu)=mom(mu,1)+mom(mu,3)
         tempv23(mu)=mom(mu,2)+mom(mu,3)
      enddo
      tempv12(4) = dotrr(tempv12,tempv12)
      tempv13(4) = dotrr(tempv13,tempv13)
      tempv23(4) = dotrr(tempv23,tempv23)


      chooseh12=abs(sqrt(tempv12(4))-sqrt(xm2(6)))! for H -> A1 A2
      chooseh13=abs(sqrt(tempv13(4))-sqrt(xm2(6))) ! for H -> A1 A3
      chooseh23=abs(sqrt(tempv23(4))-sqrt(xm2(6))) ! for H -> A2 A3


      if(chooseh12.le.min(chooseh13,chooseh23).and.chooseh12.le.7.0D-2) then
        myps = 2
      else if(chooseh13.le.min(chooseh12,chooseh23).and.chooseh13.le.7.0D-2) then
        myps = 3
      else if(chooseh23.le.min(chooseh12,chooseh13).and.chooseh23.le.7.0D-2) then
        myps = 4
      else 
        myps = 1
      endif
      if (ps_number.ne.myps) AAA_ps=.false.

      AAA_Choose_anom = AAA_ps
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION VAJ_Choose(PS_number, mom)
c*************************************************************************
c     This routine selects the right phase space generator for VVA
c     processes
c*************************************************************************
      IMPLICIT NONE

c     input variables
      integer PS_number
      double precision mom(0:3,3),tmp(0:3),tmp2(0:3)
      integer mu
      logical cond1,cond2,cond12

      do mu=0,3
      tmp(mu)=mom(mu,1)+mom(mu,2)+mom(mu,3)
      tmp2(mu)=mom(mu,1)+mom(mu,3)
      enddo
      cond1=(tmp(0)**2-tmp(1)**2-tmp(2)**2-tmp(3)**2).le.95d0**2 !continuum production of W photon
      cond2=(tmp2(0)**2-tmp2(1)**2-tmp2(2)**2-tmp2(3)**2).le.95d0**2 !coll lepton+photon is part of threebody
      cond12=cond1.and.cond2
      
      VAJ_Choose=.true.
      if (cond12 .and. PS_number.eq.2) VAJ_Choose=.false.
      if (.not.cond12 .and. PS_number.eq.1) VAJ_Choose=.false. 
      
      RETURN
      END

